/************************************************************************/
/* taxsolve_ma_1_2006.c - OpenTaxSolver for Mass Form 1 V2.0		*/
/* Copyright (C) 2006 - Robert Heller					*/
/* 									*/
/* OTS Project Home Page and Updates:  					*/
/*		http://opentaxsolver.sourceforge.com/			*/
/* 									*/
/* Compile:   cc taxsolve_ma_1_2006.c -o taxsolve_ma_1_2006		*/
/* Run:       ./taxsolve_ma_1_2006  Mass1_2006.dat			*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Robert Heller 2-10-2004	heller@deepsoft.com			*/
/* Updated 1-26-05	Aston Roberts & Robert Heller			*/
/* Updated 2-10-06	Robert Heller					*/
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include "taxsolve_routines.c"

#define SINGLE 		        1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       4
#define WIDOW		        5
#define Yes 1
#define No  0

float Sum(float *v, int slots)
{
	int i;
	float result = 0.0;
	for (i=0;i<slots;i++) result += v[i];
	return result;
}

float Compute53Tax(float taxableIncome)
{
	static struct {
		int morethan, lessthan, tax;
	} taxtable[] = {
		 {0   ,50 ,1   },
		 {50   ,100 ,4     },
		 {100   ,150 ,7   },
		 {150   ,200 ,9     },
		 {200   ,250 ,12   },
		 {250   ,300 ,15     },
		 {300   ,350 ,17   },
		 {350   ,400 ,20     },
		 {400   ,450 ,23   },
		 {450   ,500 ,25     },
		 {500   ,550 ,28   },
		 {550   ,600 ,30     },
		 {600   ,650 ,33   },
		 {650   ,700 ,36     },
		 {700   ,750 ,38   },
		 {750   ,800 ,41     },
		 {800   ,850 ,44   },
		 {850   ,900 ,46     },
		 {900   ,950 ,49   },
		 {950   ,1000 ,52     },
		 {1000   ,1050 ,54   },
		 {1050   ,1100 ,57     },
		 {1100   ,1150 ,60   },
		 {1150   ,1200 ,62     },
		 {1200   ,1250 ,65   },
		 {1250   ,1300 ,68     },
		 {1300   ,1350 ,70   },
		 {1350   ,1400 ,73     },
		 {1400   ,1450 ,76   },
		 {1450   ,1500 ,78     },
		 {1500   ,1550 ,81   },
		 {1550   ,1600 ,83     },
		 {1600   ,1650 ,86   },
		 {1650   ,1700 ,89     },
		 {1700   ,1750 ,91   },
		 {1750   ,1800 ,94     },
		 {1800   ,1850 ,97   },
		 {1850   ,1900 ,99     },
		 {1900   ,1950 ,102   },
		 {1950   ,2000 ,105     },
		 {2000   ,2050 ,107   },
		 {2050   ,2100 ,110     },
		 {2100   ,2150 ,113   },
		 {2150   ,2200 ,115     },
		 {2200   ,2250 ,118   },
		 {2250   ,2300 ,121     },
		 {2300   ,2350 ,123   },
		 {2350   ,2400 ,126     },
		 {2400   ,2450 ,129   },
		 {2450   ,2500 ,131     },
		 {2500   ,2550 ,134   },
		 {2550   ,2600 ,136     },
		 {2600   ,2650 ,139   },
		 {2650   ,2700 ,142     },
		 {2700   ,2750 ,144   },
		 {2750   ,2800 ,147     },
		 {2800   ,2850 ,150   },
		 {2850   ,2900 ,152     },
		 {2900   ,2950 ,155   },
		 {2950   ,3000 ,158     },
		 {3000   ,3050 ,160   },
		 {3050   ,3100 ,163     },
		 {3100   ,3150 ,166   },
		 {3150   ,3200 ,168     },
		 {3200   ,3250 ,171   },
		 {3250   ,3300 ,174     },
		 {3300   ,3350 ,176   },
		 {3350   ,3400 ,179     },
		 {3400   ,3450 ,182   },
		 {3450   ,3500 ,184     },
		 {3500   ,3550 ,187   },
		 {3550   ,3600 ,189     },
		 {3600   ,3650 ,192   },
		 {3650   ,3700 ,195     },
		 {3700   ,3750 ,197   },
		 {3750   ,3800 ,200     },
		 {3800   ,3850 ,203   },
		 {3850   ,3900 ,205     },
		 {3900   ,3950 ,208   },
		 {3950   ,4000 ,211     },
		 {4000   ,4050 ,213   },
		 {4050   ,4100 ,216     },
		 {4100   ,4150 ,219   },
		 {4150   ,4200 ,221     },
		 {4200   ,4250 ,224   },
		 {4250   ,4300 ,227     },
		 {4300   ,4350 ,229   },
		 {4350   ,4400 ,232     },
		 {4400   ,4450 ,235   },
		 {4450   ,4500 ,237     },
		 {4500   ,4550 ,240   },
		 {4550   ,4600 ,242     },
		 {4600   ,4650 ,245   },
		 {4650   ,4700 ,248     },
		 {4700   ,4750 ,250   },
		 {4750   ,4800 ,253     },
		 {4800   ,4850 ,256   },
		 {4850   ,4900 ,258     },
		 {4900   ,4950 ,261   },
		 {4950   ,5000 ,264	},
		 {5000 ,5050 ,266 },
		 {5050 ,5100 ,269 },
		 {5100 ,5150 ,272 },
		 {5150 ,5200 ,274 },
		 {5200 ,5250 ,277 },
		 {5250 ,5300 ,280 },
		 {5300 ,5350 ,282 },
		 {5350 ,5400 ,285 },
		 {5400 ,5450 ,288 },
		 {5450 ,5500 ,290 },
		 {5500 ,5550 ,293 },
		 {5550 ,5600 ,295 },
		 {5600 ,5650 ,298 },
		 {5650 ,5700 ,301 },
		 {5700 ,5750 ,303 },
		 {5750 ,5800 ,306 },
		 {5800 ,5850 ,309 },
		 {5850 ,5900 ,311 },
		 {5900 ,5950 ,314 },
		 {5950 ,6000 ,317 },
		 {6000 ,6050 ,319 },
		 {6050 ,6100 ,322 },
		 {6100 ,6150 ,325 },
		 {6150 ,6200 ,327 },
		 {6200 ,6250 ,330 },
		 {6250 ,6300 ,333 },
		 {6300 ,6350 ,335 },
		 {6350 ,6400 ,338 },
		 {6400 ,6450 ,341 },
		 {6450 ,6500 ,343 },
		 {6500 ,6550 ,346 },
		 {6550 ,6600 ,348 },
		 {6600 ,6650 ,351 },
		 {6650 ,6700 ,354 },
		 {6700 ,6750 ,356 },
		 {6750 ,6800 ,359 },
		 {6800 ,6850 ,362 },
		 {6850 ,6900 ,364 },
		 {6900 ,6950 ,367 },
		 {6950 ,7000 ,370 },
		 {7000 ,7050 ,372 },
		 {7050 ,7100 ,375 },
		 {7100 ,7150 ,378 },
		 {7150 ,7200 ,380 },
		 {7200 ,7250 ,383 },
		 {7250 ,7300 ,386 },
		 {7300 ,7350 ,388 },
		 {7350 ,7400 ,391 },
		 {7400 ,7450 ,394 },
		 {7450 ,7500 ,396 },
		 {7500 ,7550 ,399 },
		 {7550 ,7600 ,401 },
		 {7600 ,7650 ,404 },
		 {7650 ,7700 ,407 },
		 {7700 ,7750 ,409 },
		 {7750 ,7800 ,412 },
		 {7800 ,7850 ,415 },
		 {7850 ,7900 ,417 },
		 {7900 ,7950 ,420 },
		 {7950 ,8000 ,423 },
		 {8000 ,8050 ,425 },
		 {8050 ,8100 ,428 },
		 {8100 ,8150 ,431 },
		 {8150 ,8200 ,433 },
		 {8200 ,8250 ,436 },
		 {8250 ,8300 ,439 },
		 {8300 ,8350 ,441 },
		 {8350 ,8400 ,444 },
		 {8400 ,8450 ,447 },
		 {8450 ,8500 ,449 },
		 {8500 ,8550 ,452 },
		 {8550 ,8600 ,454 },
		 {8600 ,8650 ,457 },
		 {8650 ,8700 ,460 },
		 {8700 ,8750 ,462 },
		 {8750 ,8800 ,465 },
		 {8800 ,8850 ,468 },
		 {8850 ,8900 ,470 },
		 {8900 ,8950 ,473 },
		 {8950 ,9000 ,476 },
		 {9000 ,9050 ,478 },
		 {9050 ,9100 ,481 },
		 {9100 ,9150 ,484 },
		 {9150 ,9200 ,486 },
		 {9200 ,9250 ,489 },
		 {9250 ,9300 ,492 },
		 {9300 ,9350 ,494 },
		 {9350 ,9400 ,497 },
		 {9400 ,9450 ,500 },
		 {9450 ,9500 ,502 },
		 {9500 ,9550 ,505 },
		 {9550 ,9600 ,507 },
		 {9600 ,9650 ,510 },
		 {9650 ,9700 ,513 },
		 {9700 ,9750 ,515 },
		 {9750 ,9800 ,518 },
		 {9850 ,9900 ,523 },
		 {9950 ,10000 ,529},
		 {10000 ,10050 ,531 },
		 {10050 ,10100 ,534 },
		 {10100 ,10150 ,537 },
		 {10150 ,10200 ,539 },
		 {10200 ,10250 ,542 },
		 {10250 ,10300 ,545 },
		 {10300 ,10350 ,547 },
		 {10350 ,10400 ,550 },
		 {10400 ,10450 ,553 },
		 {10450 ,10500 ,555 },
		 {10500 ,10550 ,558 },
		 {10550 ,10600 ,560 },
		 {10600 ,10650 ,563 },
		 {10650 ,10700 ,566 },
		 {10700 ,10750 ,568 },
		 {10750 ,10800 ,571 },
		 {10800 ,10850 ,574 },
		 {10850 ,10900 ,576 },
		 {10900 ,10950 ,579 },
		 {10950 ,11000 ,582 },
		 {11000 ,11050 ,584 },
		 {11050 ,11100 ,587 },
		 {11100 ,11150 ,590 },
		 {11150 ,11200 ,592 },
		 {11200 ,11250 ,595 },
		 {11250 ,11300 ,598 },
		 {11300 ,11350 ,600 },
		 {11350 ,11400 ,603 },
		 {11400 ,11450 ,606 },
		 {11450 ,11500 ,608 },
		 {11500 ,11550 ,611 },
		 {11550 ,11600 ,613 },
		 {11600 ,11650 ,616 },
		 {11650 ,11700 ,619 },
		 {11700 ,11750 ,621 },
		 {11750 ,11800 ,624 },
		 {11800 ,11850 ,627 },
		 {11850 ,11900 ,629 },
		 {11900 ,11950 ,632 },
		 {11950 ,12000 ,635 },
		 {12000 ,12050 ,637 },
		 {12050 ,12100 ,640 },
		 {12100 ,12150 ,643 },
		 {12150 ,12200 ,645 },
		 {12200 ,12250 ,648 },
		 {12250 ,12300 ,651 },
		 {12300 ,12350 ,653 },
		 {12350 ,12400 ,656 },
		 {12400 ,12450 ,659 },
		 {12450 ,12500 ,661 },
		 {12500 ,12550 ,664 },
		 {12550 ,12600 ,666 },
		 {12600 ,12650 ,669 },
		 {12650 ,12700 ,672 },
		 {12700 ,12750 ,674 },
		 {12750 ,12800 ,677 },
		 {12800 ,12850 ,680 },
		 {12850 ,12900 ,682 },
		 {12900 ,12950 ,685 },
		 {12950 ,13000 ,688 },
		 {13000 ,13050 ,690 },
		 {13050 ,13100 ,693 },
		 {13100 ,13150 ,696 },
		 {13150 ,13200 ,698 },
		 {13200 ,13250 ,701 },
		 {13250 ,13300 ,704 },
		 {13300 ,13350 ,706 },
		 {13350 ,13400 ,709 },
		 {13400 ,13450 ,712 },
		 {13450 ,13500 ,714 },
		 {13500 ,13550 ,717 },
		 {13550 ,13600 ,719 },
		 {13600 ,13650 ,722 },
		 {13650 ,13700 ,725 },
		 {13700 ,13750 ,727 },
		 {13750 ,13800 ,730 },
		 {13800 ,13850 ,733 },
		 {13850 ,13900 ,735 },
		 {13900 ,13950 ,738 },
		 {13950 ,14000 ,741 },
		 {14000 ,14050 ,743 },
		 {14050 ,14100 ,746 },
		 {14100 ,14150 ,749 },
		 {14150 ,14200 ,751 },
		 {14200 ,14250 ,754 },
		 {14250 ,14300 ,757 },
		 {14300 ,14350 ,759 },
		 {14350 ,14400 ,762 },
		 {14400 ,14450 ,765 },
		 {14450 ,14500 ,767 },
		 {14500 ,14550 ,770 },
		 {14550 ,14600 ,772 },
		 {14600 ,14650 ,775 },
		 {14650 ,14700 ,778 },
		 {14700 ,14750 ,780 },
		 {14750 ,14800 ,783 },
		 {14800 ,14850 ,786 },
		 {14850 ,14900 ,788 },
		 {14900 ,14950 ,791 },
		 {14950 ,15000 ,794 },
		 {15000 ,15050 ,796 },
		 {15050 ,15100 ,799 },
		 {15100 ,15150 ,802 },
		 {15150 ,15200 ,804 },
		 {15200 ,15250 ,807 },
		 {15250 ,15300 ,810 },
		 {15300 ,15350 ,812 },
		 {15350 ,15400 ,815 },
		 {15400 ,15450 ,818 },
		 {15450 ,15500 ,820 },
		 {15500 ,15550 ,823 },
		 {15550 ,15600 ,825 },
		 {15600 ,15650 ,828 },
		 {15650 ,15700 ,831 },
		 {15700 ,15750 ,833 },
		 {15750 ,15800 ,836 },
		 {15800 ,15850 ,839 },
		 {15850 ,15900 ,841 },
		 {15900 ,15950 ,844 },
		 {15950 ,16000 ,847 },
		 {16000 ,16050 ,849 },
		 {16050 ,16100 ,852 },
		 {16100 ,16150 ,855 },
		 {16150 ,16200 ,857 },
		 {16200 ,16250 ,860 },
		 {16250 ,16300 ,863 },
		 {16300 ,16350 ,865 },
		 {16350 ,16400 ,868 },
		 {16400 ,16450 ,871 },
		 {16450 ,16500 ,873 },
		 {16500 ,16550 ,876 },
		 {16550 ,16600 ,878 },
		 {16600 ,16650 ,881 },
		 {16650 ,16700 ,884 },
		 {16700 ,16750 ,886 },
		 {16750 ,16800 ,889 },
		 {16800 ,16850 ,892 },
		 {16850 ,16900 ,894 },
		 {16900 ,16950 ,897 },
		 {16950 ,17000 ,900 },
		 {17000 ,17050 ,902 },
		 {17050 ,17100 ,905 },
		 {17100 ,17150 ,908 },
		 {17150 ,17200 ,910 },
		 {17200 ,17250 ,913 },
		 {17250 ,17300 ,916 },
		 {17300 ,17350 ,918 },
		 {17350 ,17400 ,921 },
		 {17400 ,17450 ,924 },
		 {17450 ,17500 ,926 },
		 {17500 ,17550 ,929 },
		 {17550 ,17600 ,931 },
		 {17600 ,17650 ,934 },
		 {17650 ,17700 ,937 },
		 {17700 ,17750 ,939 },
		 {17750 ,17800 ,942 },
		 {17800 ,17850 ,945 },
		 {17850 ,17900 ,947 },
		 {17900 ,17950 ,950 },
		 {17950 ,18000 ,953 },
		 {18000 ,18050 ,955 },
		 {18050 ,18100 ,958 },
		 {18100 ,18150 ,961 },
		 {18150 ,18200 ,963 },
		 {18200 ,18250 ,966 },
		 {18250 ,18300 ,969 },
		 {18300 ,18350 ,971 },
		 {18350 ,18400 ,974 },
		 {18400 ,18450 ,977 },
		 {18450 ,18500 ,979 },
		 {18500 ,18550 ,982 },
		 {18550 ,18600 ,984 },
		 {18600 ,18650 ,987 },
		 {18650 ,18700 ,990 },
		 {18700 ,18750 ,992 },
		 {18750 ,18800 ,995 },
		 {18800 ,18850 ,998 },
		 {18850 ,18900 ,1000 },
		 {18900 ,18950 ,1003 },
		 {18950 ,19000 ,1006 },
		 {19000 ,19050 ,1008 },
		 {19050 ,19100 ,1011 },
		 {19100 ,19150 ,1014 },
		 {19150 ,19200 ,1016 },
		 {19200 ,19250 ,1019 },
		 {19250 ,19300 ,1022 },
		 {19300 ,19350 ,1024 },
		 {19350 ,19400 ,1027 },
		 {19400 ,19450 ,1030 },
		 {19450 ,19500 ,1032 },
		 {19500 ,19550 ,1035 },
		 {19550 ,19600 ,1037 },
		 {19600 ,19650 ,1040 },
		 {19650 ,19700 ,1043 },
		 {19700 ,19750 ,1045 },
		 {19750 ,19800 ,1048 },
		 {19800 ,19850 ,1051 },
		 {19850 ,19900 ,1053 },
		 {19900 ,19950 ,1056 },
		 {19950 ,20000 ,1059 },
		 {20000 ,20050 ,1061 },
		 {20050 ,20100 ,1064 },
		 {20100 ,20150 ,1067 },
		 {20150 ,20200 ,1069 },
		 {20200 ,20250 ,1072 },
		 {20250 ,20300 ,1075 },
		 {20300 ,20350 ,1077 },
		 {20350 ,20400 ,1080 },
		 {20400 ,20450 ,1083 },
		 {20450 ,20500 ,1085 },
		 {20500 ,20550 ,1088 },
		 {20550 ,20600 ,1090 },
		 {20600 ,20650 ,1093 },
		 {20650 ,20700 ,1096 },
		 {20700 ,20750 ,1098 },
		 {20750 ,20800 ,1101 },
		 {20800 ,20850 ,1104 },
		 {20850 ,20900 ,1106 },
		 {20900 ,20950 ,1109 },
		 {20950 ,21000 ,1112 },
		 {21000 ,21050 ,1114 },
		 {21050 ,21100 ,1117 },
		 {21100 ,21150 ,1120 },
		 {21150 ,21200 ,1122 },
		 {21200 ,21250 ,1125 },
		 {21250 ,21300 ,1128 },
		 {21300 ,21350 ,1130 },
		 {21350 ,21400 ,1133 },
		 {21400 ,21450 ,1136 },
		 {21450 ,21500 ,1138 },
		 {21500 ,21550 ,1141 },
		 {21550 ,21600 ,1143 },
		 {21600 ,21650 ,1146 },
		 {21650 ,21700 ,1149 },
		 {21700 ,21750 ,1151 },
		 {21750 ,21800 ,1154 },
		 {21800 ,21850 ,1157 },
		 {21850 ,21900 ,1159 },
		 {21900 ,21950 ,1162 },
		 {21950 ,22000 ,1165 },
		 {22000 ,22050 ,1167 },
		 {22050 ,22100 ,1170 },
		 {22100 ,22150 ,1173 },
		 {22150 ,22200 ,1175 },
		 {22200 ,22250 ,1178 },
		 {22250 ,22300 ,1181 },
		 {22300 ,22350 ,1183 },
		 {22350 ,22400 ,1186 },
		 {22400 ,22450 ,1189 },
		 {22450 ,22500 ,1191 },
		 {22500 ,22550 ,1194 },
		 {22550 ,22600 ,1196 },
		 {22600 ,22650 ,1199 },
		 {22650 ,22700 ,1202 },
		 {22700 ,22750 ,1204 },
		 {22750 ,22800 ,1207 },
		 {22800 ,22850 ,1210 },
		 {22850 ,22900 ,1212 },
		 {22900 ,22950 ,1215 },
		 {22950 ,23000 ,1218 },
		 {23000 ,23050 ,1220 },
		 {23050 ,23100 ,1223 },
		 {23100 ,23150 ,1226 },
		 {23150 ,23200 ,1228 },
		 {23200 ,23250 ,1231 },
		 {23250 ,23300 ,1234 },
		 {23300 ,23350 ,1236 },
		 {23350 ,23400 ,1239 },
		 {23400 ,23450 ,1242 },
		 {23450 ,23500 ,1244 },
		 {23500 ,23550 ,1247 },
		 {23550 ,23600 ,1249 },
		 {23600 ,23650 ,1252 },
		 {23650 ,23700 ,1255 },
		 {23700 ,23750 ,1257 },
		 {23750 ,23800 ,1260 },
		 {23800 ,23850 ,1263 },
		 {23850 ,23900 ,1265 },
		 {23900 ,23950 ,1268 },
		 {23950 ,24000 ,1271 }
	};
	int i;
#define taxtable_size (sizeof(taxtable) / sizeof(taxtable[0]))

	for (i = 0; i < taxtable_size; i++) {
		if (taxableIncome >= taxtable[i].morethan &&
		    taxableIncome <  taxtable[i].lessthan) {
		    	return (float) taxtable[i].tax;
		}
	}
	return taxableIncome * .053;
			 
}

/*----------------------------------------------------------------------------*/

int main( int argc, char *argv[] )
{
 int i, j, k, status=0, i65, iblind, dep_deduct;
 int flag;
 char word[2000], outfname[2000];
 time_t now;
 float Exemptions[5], Exempt_MD, Exempt_Adoption;
 float MassBankInterest, Iexempt;
 float Unemployment, Lottery;
 float MassRetirement[2];
 float L23a, L32[5];
 float L38b;
 
 
 /* Decode any command-line arguments. */
 i = 1;  k=1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-verbose")==0)  { verbose = 1; }
  else
  if (k==1)
   {
    infile = fopen(argv[i],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[i]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[i]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,".out"); else strcpy(&(outfname[j]),".out");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname); exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else
   {printf("Unknown command-line parameter '%s'\n", argv[i]); exit(1);}
  i = i + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (i=0; i<MAX_LINES; i++) { L[i] = 0.0; }

 /* Accept parameters from input file. */
 /* Expect  US-Fed-1040 lines, something like:
	Title:  Mass Form 1 Return
	L3		{Wages}
	L4		{Taxable pensions}
	L5		{Interest}

 */

 /* Accept Form's "Title" line, and put out with date-stamp for your records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s 	%s\n", word, ctime( &now ));

 /* get_parameter(infile, kind, x, emssg ) */
 get_parameter( infile, 's', word, "Status" );	/* Single, Married/joint, Head house, Married/sep. */
 get_parameter( infile, 'l', word, "Status?");
 if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
 if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD; else
 if (strncasecmp(word,"Widow",4)==0) status = WIDOW;
 else { printf("Error: unrecognized status '%s'. Exiting.\n", word); exit(0); }
 fprintf(outfile,"Status = %s (%d)\n", word, status);

 for (i=0;i<4;i++) {Exemptions[i] = 0.0;}
 switch (status) {
   case HEAD_OF_HOUSEHOLD:
   case SINGLE: Exemptions[0] = 3850;
   	   break;
   case MARRIED_FILLING_JOINTLY: Exemptions[0] = 7700;
   	   break;
   case MARRIED_FILLING_SEPARAT: Exemptions[0] = 5950;
	   break;
 }   
 
 fprintf(outfile,"Exemptions: \n");
 fprintf(outfile,"  a. Personal exemptions = %6.2f\n", Exemptions[0]);

 get_parameter( infile, 's', word, "Dependents" );
 get_parameter( infile, 'i', &j, "Dependents"); 
 Exemptions[1] = j*1000;
 if (Exemptions[1] > 0) fprintf(outfile,"  b. Number of dependents: %d x 1,000 = %6.2f\n", j, Exemptions[1]);

 i65 = 0; iblind = 0;
 Exempt_MD = 0.0; Exempt_Adoption=0.0;
 get_parameter( infile, 's', word, "Age65You");
 get_parameter( infile, 'b', &flag, "Your age over 65?");
 if (flag) i65++;
 get_parameter( infile, 's', word, "Age65Spouse");
 get_parameter( infile, 'b', &flag, "Spouse age over 65?");
 if (flag) i65++;
 Exemptions[2] = i65*700;
 if (Exemptions[2] > 0) fprintf(outfile,"  c. Age 65 or over: %d x 700        = %6.2f\n", i65, Exemptions[2]);

 get_parameter( infile, 's', word, "BlindnessYou");
 get_parameter( infile, 'b', &flag, "Your Blindness?");
 if (flag) iblind++;
 get_parameter( infile, 's', word, "BlindnessSpouse");
 get_parameter( infile, 'b', &flag, "Spouse Blindness?");
 if (flag) iblind++;
 Exemptions[3] = iblind*2200;
 if (Exemptions[3] > 0) fprintf(outfile,"  d. Blindness: %d x 2,200        = %6.2f\n", iblind, Exemptions[3]);

/* get_parameter( infile, 's', word, "Medical/Dental");
 get_parameter( infile, 'f', &Exempt_MD, "Medical/Dental?"); */
 GetLine( "Medical/Dental", &Exempt_MD );
/* get_parameter( infile, 's', word, "Adoption");
 get_parameter( infile, 'f', &Exempt_Adoption, "Adoption?"); */
 GetLine( "Adoption", &Exempt_Adoption );

 Exemptions[4] = Exempt_MD + Exempt_Adoption;
 
 if (Exemptions[4] > 0) fprintf(outfile,"  e. Other: 1. Medical/Dental > %6.2f + 2. Adoption > %6.2f = %6.2f\n", Exempt_MD, Exempt_Adoption, Exemptions[4]);

 L[2] = Sum(Exemptions,5);
 fprintf(outfile,"  f. Total Exemptions = %6.2f\n", L[2]);
 L[18] = L[2];

 GetLine( "L3", &L[3] );	/* Wages, salery, tips (W-2). */
 showline(3);

 GetLine( "L4", &L[4] );	/* Taxable pensions. */
 ShowLine(4);

 GetLine( "L5", &MassBankInterest );
 if (status == 2) Iexempt = 200;
 else Iexempt = 100;
 L[5] = MassBankInterest - Iexempt;
 if (L[5] < 0) L[5] = 0;
 if (L[5] > 0) {
   sprintf(word,"Mass. Bank Interest: a. %6.2f - b. exemption %6.2f",
	MassBankInterest,Iexempt);
   showline_wmsg(5,word);
 }

 GetLine( "L6", &L[6] );	/* Business income/loss. */
 ShowLine(6);

 GetLine( "L7", &L[7] );	/* Rental, royality, REMIC. */
 ShowLine(7);

 GetLine( "L8a", &Unemployment );	/* Unemployment */
 GetLine( "L8b", &Lottery );	/* Lottery */
 L[8] = Unemployment + Lottery;
 if (L[8] > 0) {
   sprintf(word,"a. %6.2f + b. %6.2f",Unemployment,Lottery);
   showline_wmsg(8,word);
 }

 GetLine( "L9", &L[9] );	/* Alimony received. */
 ShowLine(9);

 L[10] = Sum(&L[3],7);
 ShowLine(10);
 
 /* Amount paid to SS, Medicare, RR, US, or Mass retirement */
 GetLine( "L11a", &MassRetirement[0] ); /* You */
 if (MassRetirement[0] > 2000) MassRetirement[0] = 2000;
 GetLine( "L11b", &MassRetirement[1] ); /* Spouse */
 if (MassRetirement[1] > 2000) MassRetirement[1] = 2000;
 L[11] = Sum(MassRetirement,2);
 if (L[11] > 0) {
   sprintf(word,"you %6.2f + spouse %6.2f",MassRetirement[0],MassRetirement[1]);
   showline_wmsg(11,word);
 }

 GetLine( "L12", &L[12] ); /* Child under 13... */
 ShowLine(12);

 get_parameter( infile, 's', word, "L13"); /* mumble...*/
 get_parameter( infile, 'i', &dep_deduct, "L13");
 if (dep_deduct > 2) dep_deduct = 2;
 if (L[12] == 0 && status != 4 && dep_deduct > 0) {
   L[13] = dep_deduct * 3600;
   sprintf(word,"a. %d x 3,600",dep_deduct);
   showline_wmsg(13,word);
 }
 GetLine( "L14", &L[14] ); /* Rental deduction */
 L[14] = L[14] / 2;
 if (status == 4) {
   if (L[14] > 1500) {L[14] = 1500;}
 } else {
   if (L[14] > 3000) {L[14] = 3000;}
 }
 ShowLine(14);

 GetLine( "L15", &L[15] ); /* Other Deductions (sched Y, L9) */
 ShowLine(15);

 L[16] = Sum(&L[11],5);

 showline_wmsg(16,"Total Deductions");

 L[17] = L[10] - L[16];
 if (L[17] < 0) L[17] = 0;

 showline(17);

 showline(18);

 L[19] = L[17] - L[18];
 if (L[19] < 0) L[19] = 0;

 GetLine( "L20", &L[20] ); /* Interest and Dividends -- Sched B */
 ShowLine(20);

 L[21] = L[19] + L[20];

 showline_wmsg(21,"Total 5.3 taxable income");
 
 L[22] = Compute53Tax(L[21]);
 showline_wmsg(22,"5.3% Tax");

 GetLine( "L23a", &L23a ); /* 12% income */

 L[23] = L23a * .12;
 if (L23a > 0) {
 	sprintf(word,"12%% Income tax: a. %6.2f x .12",L23a);
 	showline_wmsg(23,word);
 }

 GetLine( "L24", &L[24] ); /* Tax on long-term capital gains, sched D */
 ShowLine(24);

 GetLine( "L25", &L[25] ); /* Credit Capture amount Sch. H-2 */
 ShowLine(25);

 L[27] = Sum(&L[22],4);
 showline_wmsg(27,"Total Tax");

 GetLine1( "L28", &L[28] ); /* Limited Income Credit */
 ShowLine(28);

 GetLine1( "L29", &L[29] ); /* Other credits from Sch Z, line 3 */
 ShowLine(29);

 L[30] = L[28]+L[29];
 ShowLine(30);

 L[31] = L[27] - L[30];
 if (L[31] < 0) L[31] = 0;
 showline_wmsg(31,"Tax after credits");

 GetLine1( "L32a", &L32[0] ); /* Organ Transplant */
 GetLine1( "L32b", &L32[1] ); /* Endangered Wildlife */
 GetLine1( "L32c", &L32[2] ); /* Mass AIDS */
 GetLine1( "L32d", &L32[3] ); /* Mass US Olympic */
 GetLine1( "L32e", &L32[4] ); /* Mass Military Family Relief */
 
 L[32] = Sum(L32,5);
 if (L[32] > 0) {
 	sprintf(word,"a. %6.2f + b. %6.2f + c. %6.2f + d. %6.2f + 3. %6.2f = ",
 		L32[0],L32[1],L32[2],L32[3],L32[4]);
 	showline_wmsg(32,word);
 }
 GetLine1( "L33", &L[33] ); /* Use tax due on out-of-state purchases */
 ShowLine(33);

 L[34] = Sum(&L[31],3);
 showline_wmsg(34,"Tax less credits plus contribs");

 
 /* Payments section. */

 GetLine( "L35", &L[35] );	/* Mass income tax withheld, Forms W-2, 1099 */
 ShowLine(35);

 GetLine( "L36", &L[36] );	/* 2005 overpayment applied to 2006 estimated tax */
 ShowLine(36);

 GetLine( "L37", &L[37] );	/* 2006 estimated tax payments */
 ShowLine(37);



 GetLine1( "L38", &L38b );	/* Earned income credit (EIC) */
 L[38] = L38b * .15;
 ShowLine(38);

 GetLine1( "L39", &L[39] );	/* Senior Circuit Breaker Credit, sched CB */
 ShowLine(39);

 GetLine( "L40", &L[40] );	/* Payments made with extension */
 ShowLine(40);

 for (i=35; i<=40; i++) L[41] = L[41] + L[i];
 showline_wmsg(41,"total payments");


 /* Refund or Owe section. */

 if (L[41]>L[34]) 
  {
   L[42] = L[41] - L[34];
   fprintf(outfile,"L42 = %6.2f  REBATE!!!\n", L[42] );
   GetLine1( "L43", &L[43] );	/* Overpayment to be applied to 2006 estimated tax */
   if (L[43] > L[42]) L[43] = L[42];
   showline_wmsg(43,"Overpayment to be applied to 2006 estimated tax");
   L[44] = L[42] - L[43];
   fprintf(outfile,"L44 = %6.2f  THIS IS YOUR REFUND\n",L[44]);
  }
 else 
  {
   L[45] = L[34] - L[41];
   fprintf(outfile,"L45 = %6.2f  YOU OWE !!!\n", L[45] );
  }
 

 fclose(infile);
 fclose(outfile);

 printf("\nListing results from file: %s\n\n", outfname);
 Display_File( outfname );

 return 0;
}
